# -*- coding: UTF-8 -*-
###########################################################################
# Eole NG - 2007
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr
#
# Librairie Python pour EclairNg
#
###########################################################################
from tempfile import mktemp
from crypt import crypt
from os import system

def system_search(user, etcfile):
    """
    recherche d'un login dans un fichier systeme
    """
    passfile = open('/etc/%s' % etcfile,'r')
    passbuffer = passfile.readlines()
    passfile.close()
    for ligne in passbuffer:
        if user == ligne.split(':')[0]:
            return True
    return False

def is_system_user(user):
    """
    indique si le login proposé est déjà un utilisateur système
    """
    user = user.lower()
    return system_search(user, 'passwd')

def is_system_group(group):
    """
    indique si le groupe proposé est déjà un groupe système
    """
    group = group.lower()
    return system_search(group, 'group')

def add_group(group):
    """
    ajout d'un groupe
    """
    print "ajout du groupe %s" % group
    system("""/usr/sbin/groupadd "%s" """ % group)

def add_user(login, group, pwd, gecos=''):
    """
    ajout d'un utilisateur
    """
    if not is_system_group(group):
        add_group(group)
    pcrypt = crypt(pwd, "zz")
    system("""/usr/sbin/useradd -c "%s" -g "%s" -m -p "%s" "%s" """ % (gecos, group, pcrypt, login))
    print "ajout de l'utilisateur %s/%s" % (login, group)
    print "avec le mot de passe %s" % pwd

def inscription(user, groups):
    """
    inscription d'un utilisateur à une liste de groupes
    """
    if type(groups) != list:
        groups = [groups]
    for group in groups:
        if not is_system_group(group):
            add_group(group)
        print "Ajout de l'utilisatteur %s dans le groupe %s" % (user, group)
        system('/usr/sbin/adduser %s %s' % (user, group))

def gen_date(date):
    """
    formattage de la date de naissance
    """
    date = date.replace('/','')
    date = date.replace('-','')
    date = date.replace(' ','')
    if len(date) != 8:
        return '00000000'
    else:
        return '%s%s%s' % (date[4:8], date[2:4], date[0:2])

def replace_cars(chaine):
    """
    fonction de suppression des caractères spéciaux les plus courants
    Arg !
    """
    chars = {"¥" : "Y", "µ" : "u", "À" : "A", "Á" : "A",
                "Â" : "A", "Ã" : "A", "Ä" : "A", "Å" : "A",
                "Æ" : "A", "Ç" : "C", "È" : "E", "É" : "E",
                "Ê" : "E", "Ë" : "E", "Ì" : "I", "Í" : "I",
                "Î" : "I", "Ï" : "I", "Ð" : "D", "Ñ" : "N",
                "Ò" : "O", "Ó" : "O", "Ô" : "O", "Õ" : "O",
                "Ö" : "O", "Ø" : "O", "Ù" : "U", "Ú" : "U",
                "Û" : "U", "Ü" : "U", "Ý" : "Y", "ß" : "s",
                "à" : "a", "á" : "a", "â" : "a", "ã" : "a",
                "ä" : "a", "å" : "a", "æ" : "a", "ç" : "c",
                "è" : "e", "é" : "e", "ê" : "e", "ë" : "e",
                "ì" : "i", "í" : "i", "î" : "i", "ï" : "i",
                "ð" : "o", "ñ" : "n", "ò" : "o", "ó" : "o",
                "ô" : "o", "õ" : "o", "ö" : "o", "ø" : "o",
                "ù" : "u", "ú" : "u", "û" : "u", "ü" : "u",
                "ý" : "y", "ÿ" : "y", "~B": "e"}
    for old, new in chars.items():
        chaine = chaine.replace(old, new)
    return chaine

def manage_nom(nom):
    """ gestion des noms à particule """
    nom = nom.lower().replace("'", '')
    particules = ['de', 'du', 'dela', 'dos', 'di', 'es', 'el', 'le', 'la', 'da', 'van', 'ben', 'saint']
    for particule in particules:
        if nom.startswith('%s ' % particule):
            nom = nom.replace('%s ' % particule, particule, 1)
    nom = nom.split(' ')[0]
    nom = nom.split('-')[0]
    return nom

def gen_login(prenom, nom):
    """
    génération d'un login acceptable :)
    """
    prenom = replace_cars(prenom.replace('-', '').split(' ')[0]).lower()
    nom = replace_cars(manage_nom(nom)).lower()
    if prenom != '':
        login = "%s.%s" % (prenom, nom)
    else:
        login = nom
    if len(login) > 20:
        login = login[0:20]
    return login.replace(' ','')

def ok_groupe(groupe, tgroup='g'):
    """
    génération d'un nom de groupe acceptable :)
    """
    groupe = groupe.split()[0].lower()
    groupe = groupe.split('(')[0]     #parentheses
    groupe = groupe.replace(' ', '')  #espaces
    groupe = groupe.replace('*', 'e') #etoile, ex : mp*
    groupe = groupe.replace('.', '')  #point
    groupe = groupe.replace(':', '')  #2points
    groupe = groupe.replace('=', '')  #égal
    groupe = groupe.replace('"', '')  #double-quotes
    groupe = groupe.replace("'", '')  #quote
    groupe = replace_cars(groupe)
    # groupes 100% numériques
    #if groupe.isdigit():
    #    groupe = "g"+groupe
    if groupe[0].isdigit():
        groupe = "%s%s" % (tgroup, groupe)
    if len(groupe) > 19:
        groupe = groupe[0:20]
    return groupe

def convert_file(filename):
    """
    conversion du fichier en UTF-8
    (si nécessaire)
    """
    cmd = '/usr/bin/iconv'
    tmpf = mktemp()
    cont = file(filename, 'r').read()
    try:
        unicode(cont, 'utf8')
        # déjà en UTF-8
        return True
    except:
        try:
            # fichier en ISO
            unicode(cont, 'iso8859-1')
            system('%s -f iso8859-1 -t utf8 %s > %s' % (cmd, filename, tmpf))
            system('/bin/mv -f %s %s' % (tmpf, filename))
        except:
            return False
    return True

