# -*- coding: UTF-8 -*-
###########################################################################
# Eole NG - 2007
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr
#
# Parseur pour fichies Sconet, STS, AAF, Be1d
#
###########################################################################
import xml.dom.minidom
import backend
from os import popen
import csv

def get_text_node(node):
    """
    @param node: node minidom contenant du texte
    Utilitaire minidom permettant de récupérer le texte d'un node texte
    """
    texte = ""
    nodelist = node.childNodes
    for textnode in nodelist:
        if textnode.nodeType == textnode.TEXT_NODE:
            texte = textnode.data
    return texte

def parse_nomenclatures(nomenclatures):
    """
    récupération des MATIERES
    renvoit un dictionnaire : code->nom
    """
    code_matiere = {}
    for mat in nomenclatures.getElementsByTagName('MATIERE'):
        code_mat = mat.getAttribute('CODE')
        code_gestion = mat.getElementsByTagName('CODE_GESTION')[0]
        code_matiere[code_mat] = get_text_node(code_gestion)
    return code_matiere

def parse_divisions(divisions, code_matiere, enseigne):
    """
    récupération des DIVISIONS avec MATIERES et ENSEIGNANTS
    """
    for division in divisions.getElementsByTagName('DIVISION'):
        classe = division.getAttribute('CODE')
        classe = backend.ok_groupe(classe, 'c')
        for service in division.getElementsByTagName('SERVICE'):
            type_cours = service.getAttribute('CODE_MOD_COURS')
            matiere = code_matiere[service.getAttribute('CODE_MATIERE')]
            matiere = backend.ok_groupe(matiere, 'm')
            for enseignant in service.getElementsByTagName('ENSEIGNANT'):
                ens = enseignant.getAttribute('ID')
                if enseigne.has_key(ens):
                    if classe not in enseigne[ens]:
                        enseigne[ens].append(classe)
                    if matiere not in enseigne[ens]:
                        enseigne[ens].append(matiere)
                else:

                    enseigne[ens] = [classe, matiere]
    return enseigne

def parse_groupes(groupes, code_matiere, enseigne):
    """
    récupération des GROUPES avec ENSEIGNANTS et CLASSE
    """
    for groupe in groupes.getElementsByTagName('GROUPE'):
        appartenance = []
        for division in groupe.getElementsByTagName('DIVISION_APPARTENANCE'):
            classe = backend.ok_groupe(division.getAttribute('CODE'), 'c')
            appartenance.append(classe)
        for service in groupe.getElementsByTagName('SERVICE'):
            matiere = code_matiere[service.getAttribute('CODE_MATIERE')]
            matiere = backend.ok_groupe(matiere, 'm')
            for enseignant in service.getElementsByTagName('ENSEIGNANT'):
                ens = enseignant.getAttribute('ID')
                for classe in appartenance:
                    if enseigne.has_key(ens):
                        if classe not in enseigne[ens]:
                            enseigne[ens].append(classe)
                        if matiere not in enseigne[ens]:
                            enseigne[ens].append(matiere)
                    else:
                        enseigne[ens] = [classe, matiere]
    return enseigne

def parse_sts_file(filename, pwdtype="1", passfile="/tmp/passwords.csv"):
    """
    parsing du fichier XML STS-Web (professeurs) :
    filename : fichier xml source
    pwdtype  : "1" pour date de naissance, "2" pour aléatoire
    passfile  : fichier résultat
    """
    enseigne = {}

    try:
        document = xml.dom.minidom.parse(filename)
    except:
        return ["Fichier invalide"]

    root_node = document.documentElement

    # recuperation des matieres
    code_matiere = parse_nomenclatures(root_node.getElementsByTagName('NOMENCLATURES')[0])

    # récupération des classes et des matières
    enseigne = parse_divisions(root_node.getElementsByTagName('DIVISIONS')[0], code_matiere, enseigne)

    # affectation des groupes
    enseigne = parse_groupes(root_node.getElementsByTagName('GROUPES')[0], code_matiere, enseigne)

    individus = root_node.getElementsByTagName('INDIVIDUS')[0]
    for individu in individus.getElementsByTagName('INDIVIDU'):
        tab = {}
        ident = individu.getAttribute('ID')
        # tags non utilisés
        # PROF_PRINC, CIVILITE, DISCIPLINE

        # autres champs
        dico = {}
        for champ in ['NOM_USAGE','PRENOM','DATE_NAISSANCE']:
            try:
                dico[champ] = get_text_node(individu.getElementsByTagName(champ)[0])
            except:
                dico[champ] = ''
        login = backend.gen_login(dico['PRENOM'], dico['NOM_USAGE'])
        if pwdtype == '1' and dico.has_key('DATE_NAISSANCE'):
            pwd = backend.gen_date(dico['DATE_NAISSANCE'])
        else:
            pwd = popen("/usr/bin/pwgen -1").read().strip()

        if backend.is_system_user(login):
            print "Utilisateur déjà existant"
            #messages.append("Utilisateur %s déjà existant"%login)
            continue
        # FIXME
        print
        backend.add_user(login, 'professeurs', pwd, "%(PRENOM)s %(NOM_USAGE)s" % dico)
        if enseigne.has_key(ident):
            backend.inscription(login, enseigne[ident])
        #passlines.append("%s;%s\n"%(login, date))
        #if (valide != 0 and valide%50  == 0):
        #    print "<font size=-1>%d professeurs lus</font><br>"%valide

def parse_sco_file(filename, pwdtype="1", passfile="/tmp/passwords.csv"):
    """
    parsing du fichier CVS Sconet (élèves) :
    filename : fichier csv source
    pwdtype  : "1" pour date de naissance, "2" pour aléatoire
    passfile  : fichier résultat
    """
    fic = open(filename)
    lignes = fic.read().split()
    fic.close()
    # attributs obligatoires et indexation
    mandatory = ["Nom", u"Prénom 1", "Date de naissance", "Sexe", "MEF", u"N° Interne", "Division"]
    index = {}
    attrs = lignes[0].split()
    for mandat in mandatory:
        if mandat not in attrs:
            print "Le champ %s est requis !" % mandat
            return
        index[mandat] = attrs.index(mandat)
    # parcours des données
    for datas in lignes[1:]:
        data = datas.split(';')
        if len(data) < mini:
            print "ligne %s invalide" % data[index[u"N° Interne"]]
            continue
        num = data[index[u"N° Interne"]].strip()
        if num == '':
            continue
        prenom = data[index[u"Prénom 1"]]
        nom = data[index["Nom"]]
        login = backend.gen_login(prenom, nom)
        if pwdtype == '1':
            pwd = backend.gen_date(data[index["Date de naissance"]])
        else:
            pwd = popen("/usr/bin/pwgen -1").read().strip()

        if backend.is_system_user(login):
            print "Utilisateur déjà existant"
            #messages.append("Utilisateur %s déjà existant"%login)
            continue
        print
        backend.add_user(login, 'eleves', pwd, "%s %s" % (prenom, nom))
        classe = backend.ok_groupe(data[index["Division"]], 'c')
        backend.inscription(login, classe)
        niveau = backend.ok_groupe(data[index["MEF"]], 'n')
        backend.inscription(login, classe)

def parse_be1d_file(filename, pwdtype="1", passfile="/tmp/passwords.csv"):
    """
    parsing du fichier CSV Be1d (élèves) :
    filename : fichier csv source
    pwdtype  : "1" pour date de naissance, "2" pour aléatoire
    passfile  : fichier résultat
    """

    # champs obligatoires
    fields = ['Nom Elève','Prénom Elève','Date naissance','Sexe','Niveau','Classe']

    # conversion du fichier en UTF-8 si nécessaire
    backend.convert_file(filename)
    reader = csv.DictReader(file(filename), delimiter=';')

    # validation en-tête
    head = reader.next()
    for field in fields:
        if not head.has_key(field):
            print "Champ %s manquant !" % field
            return False
    while True:
        try:
            data = reader.next()
            prenom = data['Prénom Elève']
            nom = data['Nom Elève']
            login = backend.gen_login(prenom, nom)
            if pwdtype == '1':
                pwd = backend.gen_date(data['Date naissance'])
            else:
                pwd = popen("/usr/bin/pwgen -1").read().strip()

            if backend.is_system_user(login):
                print "Utilisateur %s déjà existant" % login
                continue
            print
            backend.add_user(login, 'eleves', pwd, "%s %s" % (prenom, nom))
            classe = backend.ok_groupe(data['Classe'], 'c')
            backend.inscription(login, classe)
            niveau = backend.ok_groupe(data['Niveau'], 'n')
            backend.inscription(login, classe)
        except StopIteration:
            break

